@@ -74,7 +74,6 @@ class BaseCinderAPIResourceWrapper(base.APIResourceWrapper):
     def name(self):
         # If a volume doesn't have a name, use its id.
         return (getattr(self._apiresource, 'name', None) or
-                getattr(self._apiresource, 'display_name', None) or
                 getattr(self._apiresource, 'id', None))
 
     @property
@@ -88,9 +87,10 @@ class Volume(BaseCinderAPIResourceWrapper):
     _attrs = ['id', 'name', 'description', 'size', 'status', 'created_at',
               'volume_type', 'availability_zone', 'imageRef', 'bootable',
               'snapshot_id', 'source_volid', 'attachments', 'tenant_name',
-              'consistencygroup_id', 'os-vol-host-attr:host',
+              'group_id', 'consistencygroup_id', 'os-vol-host-attr:host',
               'os-vol-tenant-attr:tenant_id', 'metadata',
-              'volume_image_metadata', 'encrypted', 'transfer']
+              'volume_image_metadata', 'encrypted', 'transfer',
+              'multiattach']
 
     @property
     def is_bootable(self):
@@ -100,7 +100,7 @@ class Volume(BaseCinderAPIResourceWrapper):
 class VolumeSnapshot(BaseCinderAPIResourceWrapper):
 
     _attrs = ['id', 'name', 'description', 'size', 'status',
-              'created_at', 'volume_id',
+              'created_at', 'volume_id', 'group_snapshot_id',
               'os-extended-snapshot-attributes:project_id',
               'metadata']
 
@@ -172,6 +172,21 @@ class VolumePool(base.APIResourceWrapper):
               'storage_protocol', 'extra_specs']
 
 
+class Group(base.APIResourceWrapper):
+    _attrs = ['id', 'status', 'availability_zone', 'created_at', 'name',
+              'description', 'group_type', 'volume_types',
+              'group_snapshot_id', 'source_group_id', 'replication_status']
+
+
+class GroupSnapshot(base.APIResourceWrapper):
+    _attrs = ['id', 'name', 'description', 'status', 'created_at',
+              'group_id', 'group_type_id']
+
+
+class GroupType(base.APIResourceWrapper):
+    _attrs = ['id', 'name', 'description', 'is_public', 'group_specs']
+
+
 def get_auth_params_from_request(request):
     auth_url = base.url_for(request, 'identity')
     cinder_urls = []
@@ -248,13 +263,11 @@ def get_microversion(request, features):
         'cinder', features, api_versions.APIVersion, min_ver, max_ver))
 
 
-def _replace_v2_parameters(data):
-    if VERSIONS.active < 2:
-        data['display_name'] = data['name']
-        data['display_description'] = data['description']
-        del data['name']
-        del data['description']
-    return data
+def _cinderclient_with_generic_groups(request):
+    version = get_microversion(request, 'groups')
+    if version is not None:
+        version = version.get_string()
+    return cinderclient(request, version=version)
 
 
 def version_get():
@@ -298,7 +311,8 @@ def volume_list_paged(request, search_opts=None, marker=None, paginate=False,
     has_prev_data = False
     volumes = []
 
-    c_client = cinderclient(request)
+    # To support filtering with group_id, we need to use the microversion.
+    c_client = _cinderclient_with_generic_groups(request)
     if c_client is None:
         return volumes, has_more_data, has_prev_data
 
@@ -330,7 +344,8 @@ def volume_list_paged(request, search_opts=None, marker=None, paginate=False,
 
 @profiler.trace
 def volume_get(request, volume_id):
-    volume_data = cinderclient(request).volumes.get(volume_id)
+    client = _cinderclient_with_generic_groups(request)
+    volume_data = client.volumes.get(volume_id)
 
     for attachment in volume_data.attachments:
         if "server_id" in attachment:
@@ -355,7 +370,9 @@ def volume_get(request, volume_id):
 @profiler.trace
 def volume_create(request, size, name, description, volume_type,
                   snapshot_id=None, metadata=None, image_id=None,
-                  availability_zone=None, source_volid=None):
+                  availability_zone=None, source_volid=None,
+                  group_id=None):
+    client = _cinderclient_with_generic_groups(request)
     data = {'name': name,
             'description': description,
             'volume_type': volume_type,
@@ -363,10 +380,10 @@ def volume_create(request, size, name, description, volume_type,
             'metadata': metadata,
             'imageRef': image_id,
             'availability_zone': availability_zone,
-            'source_volid': source_volid}
-    data = _replace_v2_parameters(data)
+            'source_volid': source_volid,
+            'group_id': group_id}
 
-    volume = cinderclient(request).volumes.create(size, **data)
+    volume = client.volumes.create(size, **data)
     return Volume(volume)
 
 
@@ -397,7 +414,6 @@ def volume_set_bootable(request, volume_id, bootable):
 def volume_update(request, volume_id, name, description):
     vol_data = {'name': name,
                 'description': description}
-    vol_data = _replace_v2_parameters(vol_data)
     return cinderclient(request).volumes.update(volume_id,
                                                 **vol_data)
 
@@ -414,7 +430,7 @@ def volume_delete_metadata(request, volume_id, keys):
 
 @profiler.trace
 def volume_reset_state(request, volume_id, state):
-    return cinderclient(request).volumes.reset_state(volume_id, state)
+    cinderclient(request).volumes.reset_state(volume_id, state)
 
 
 @profiler.trace
@@ -443,7 +459,8 @@ def volume_migrate(request, volume_id, host, force_host_copy=False,
 
 @profiler.trace
 def volume_snapshot_get(request, snapshot_id):
-    snapshot = cinderclient(request).volume_snapshots.get(snapshot_id)
+    client = _cinderclient_with_generic_groups(request)
+    snapshot = client.volume_snapshots.get(snapshot_id)
     return VolumeSnapshot(snapshot)
 
 
@@ -461,7 +478,7 @@ def volume_snapshot_list_paged(request, search_opts=None, marker=None,
     has_more_data = False
     has_prev_data = False
     snapshots = []
-    c_client = cinderclient(request)
+    c_client = _cinderclient_with_generic_groups(request)
     if c_client is None:
         return snapshots, has_more_data, has_more_data
 
@@ -492,7 +509,6 @@ def volume_snapshot_create(request, volume_id, name,
     data = {'name': name,
             'description': description,
             'force': force}
-    data = _replace_v2_parameters(data)
 
     return VolumeSnapshot(cinderclient(request).volume_snapshots.create(
         volume_id, **data))
@@ -507,7 +523,6 @@ def volume_snapshot_delete(request, snapshot_id):
 def volume_snapshot_update(request, snapshot_id, name, description):
     snapshot_data = {'name': name,
                      'description': description}
-    snapshot_data = _replace_v2_parameters(snapshot_data)
     return cinderclient(request).volume_snapshots.update(snapshot_id,
                                                          **snapshot_data)
 
@@ -742,7 +757,7 @@ def volume_manage(request,
                   metadata,
                   bootable):
     source = {id_type: identifier}
-    return cinderclient(request).volumes.manage(
+    cinderclient(request).volumes.manage(
         host=host,
         ref=source,
         name=name,
@@ -1106,3 +1121,162 @@ def volume_type_add_project_access(request, volume_type, project_id):
 def volume_type_remove_project_access(request, volume_type, project_id):
     return cinderclient(request).volume_type_access.remove_project_access(
         volume_type, project_id)
+
+
+@profiler.trace
+def group_type_list(request):
+    client = _cinderclient_with_generic_groups(request)
+    return [GroupType(t) for t in client.group_types.list()]
+
+
+@profiler.trace
+def group_type_get(request, group_type_id):
+    client = _cinderclient_with_generic_groups(request)
+    return GroupType(client.group_types.get(group_type_id))
+
+
+@profiler.trace
+def group_type_create(request, name, description=None, is_public=None):
+    client = _cinderclient_with_generic_groups(request)
+    params = {'name': name}
+    if description is not None:
+        params['description'] = description
+    if is_public is not None:
+        params['is_public'] = is_public
+    return GroupType(client.group_types.create(**params))
+
+
+@profiler.trace
+def group_type_update(request, group_type_id, data):
+    client = _cinderclient_with_generic_groups(request)
+    return GroupType(client.group_types.update(group_type_id, **data))
+
+
+@profiler.trace
+def group_type_delete(request, group_type_id):
+    client = _cinderclient_with_generic_groups(request)
+    client.group_types.delete(group_type_id)
+
+
+@profiler.trace
+def group_type_spec_set(request, group_type_id, metadata):
+    client = _cinderclient_with_generic_groups(request)
+    client.group_types.set_keys(metadata)
+
+
+@profiler.trace
+def group_type_spec_unset(request, group_type_id, keys):
+    client = _cinderclient_with_generic_groups(request)
+    client.group_types.unset_keys(keys)
+
+
+@profiler.trace
+def group_list(request, search_opts=None):
+    client = _cinderclient_with_generic_groups(request)
+    return [Group(g) for g in client.groups.list(search_opts=search_opts)]
+
+
+@profiler.trace
+def group_list_with_vol_type_names(request, search_opts=None):
+    groups = group_list(request, search_opts)
+    vol_types = volume_type_list(request)
+    for group in groups:
+        group.volume_type_names = []
+        for vol_type_id in group.volume_types:
+            for vol_type in vol_types:
+                if vol_type.id == vol_type_id:
+                    group.volume_type_names.append(vol_type.name)
+                    break
+
+    return groups
+
+
+@profiler.trace
+def group_get(request, group_id):
+    client = _cinderclient_with_generic_groups(request)
+    group = client.groups.get(group_id)
+    return Group(group)
+
+
+@profiler.trace
+def group_get_with_vol_type_names(request, group_id):
+    group = group_get(request, group_id)
+    vol_types = volume_type_list(request)
+    group.volume_type_names = []
+    for vol_type_id in group.volume_types:
+        for vol_type in vol_types:
+            if vol_type.id == vol_type_id:
+                group.volume_type_names.append(vol_type.name)
+                break
+    return group
+
+
+@profiler.trace
+def group_create(request, name, group_type, volume_types,
+                 description=None, availability_zone=None):
+    client = _cinderclient_with_generic_groups(request)
+    params = {'name': name,
+              'group_type': group_type,
+              # cinderclient expects a comma-separated list of volume types.
+              'volume_types': ','.join(volume_types)}
+    if description is not None:
+        params['description'] = description
+    if availability_zone is not None:
+        params['availability_zone'] = availability_zone
+    return Group(client.groups.create(**params))
+
+
+@profiler.trace
+def group_create_from_source(request, name, group_snapshot_id=None,
+                             source_group_id=None, description=None,
+                             user_id=None, project_id=None):
+    client = _cinderclient_with_generic_groups(request)
+    return Group(client.groups.create_from_src(
+        group_snapshot_id, source_group_id, name, description,
+        user_id, project_id))
+
+
+@profiler.trace
+def group_delete(request, group_id, delete_volumes=False):
+    client = _cinderclient_with_generic_groups(request)
+    client.groups.delete(group_id, delete_volumes)
+
+
+@profiler.trace
+def group_update(request, group_id, name=None, description=None,
+                 add_volumes=None, remove_volumes=None):
+    data = {}
+    if name is not None:
+        data['name'] = name
+    if description is not None:
+        data['description'] = description
+    if add_volumes:
+        # cinderclient expects a comma-separated list of volume types.
+        data['add_volumes'] = ','.join(add_volumes)
+    if remove_volumes:
+        # cinderclient expects a comma-separated list of volume types.
+        data['remove_volumes'] = ','.join(remove_volumes)
+    client = _cinderclient_with_generic_groups(request)
+    return client.groups.update(group_id, **data)
+
+
+def group_snapshot_create(request, group_id, name, description=None):
+    client = _cinderclient_with_generic_groups(request)
+    return GroupSnapshot(client.group_snapshots.create(group_id, name,
+                                                       description))
+
+
+def group_snapshot_get(request, group_snapshot_id):
+    client = _cinderclient_with_generic_groups(request)
+    return GroupSnapshot(client.group_snapshots.get(group_snapshot_id))
+
+
+def group_snapshot_list(request, search_opts=None):
+    client = _cinderclient_with_generic_groups(request)
+    return [GroupSnapshot(s) for s
+            in client.group_snapshots.list(search_opts=search_opts)]
+
+
+def group_snapshot_delete(request, group_snapshot_id):
+    client = _cinderclient_with_generic_groups(request)
+    client.group_snapshots.delete(group_snapshot_id)
