@@ -25,6 +25,8 @@ from django.utils.translation import ugettext_lazy as _
 import six
 import six.moves.urllib.parse as urlparse
 
+from keystoneauth1 import session
+from keystoneauth1 import token_endpoint
 from keystoneclient import exceptions as keystone_exceptions
 
 from openstack_auth import backend
@@ -154,7 +156,7 @@ def keystoneclient(request, admin=False):
     The client is cached so that subsequent API calls during the same
     request/response cycle don't have to be re-authenticated.
     """
-    api_version = VERSIONS.get_active_version()
+    client_version = VERSIONS.get_active_version()
     user = request.user
     token_id = user.token.id
 
@@ -184,21 +186,29 @@ def keystoneclient(request, admin=False):
         conn = getattr(request, cache_attr)
     else:
         endpoint = _get_endpoint_url(request, endpoint_type)
-        insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
+        verify = not getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
         cacert = getattr(settings, 'OPENSTACK_SSL_CACERT', None)
+        verify = verify and cacert
         LOG.debug("Creating a new keystoneclient connection to %s.", endpoint)
         remote_addr = request.environ.get('REMOTE_ADDR', '')
-        conn = api_version['client'].Client(token=token_id,
-                                            endpoint=endpoint,
-                                            original_ip=remote_addr,
-                                            insecure=insecure,
-                                            cacert=cacert,
-                                            auth_url=endpoint,
-                                            debug=settings.DEBUG)
+        token_auth = token_endpoint.Token(endpoint=endpoint,
+                                          token=token_id)
+        keystone_session = session.Session(auth=token_auth,
+                                           original_ip=remote_addr,
+                                           verify=verify)
+        conn = client_version['client'].Client(session=keystone_session,
+                                               debug=settings.DEBUG)
         setattr(request, cache_attr, conn)
     return conn
 
 
+@profiler.trace
+def get_identity_api_version(request):
+    client = keystoneclient(request)
+    endpoint_data = client.session.get_endpoint_data(service_type='identity')
+    return endpoint_data.api_version
+
+
 @profiler.trace
 def domain_create(request, name, description=None, enabled=None):
     manager = keystoneclient(request, admin=True).domains
@@ -216,7 +226,7 @@ def domain_get(request, domain_id):
 @profiler.trace
 def domain_delete(request, domain_id):
     manager = keystoneclient(request, admin=True).domains
-    return manager.delete(domain_id)
+    manager.delete(domain_id)
 
 
 @profiler.trace
@@ -348,7 +358,7 @@ def tenant_get(request, project, admin=True):
 @profiler.trace
 def tenant_delete(request, project):
     manager = VERSIONS.get_project_manager(request, admin=True)
-    return manager.delete(project)
+    manager.delete(project)
 
 
 @profiler.trace
@@ -451,7 +461,7 @@ def user_create(request, name=None, email=None, password=None, project=None,
 
 @profiler.trace
 def user_delete(request, user_id):
-    return keystoneclient(request, admin=True).users.delete(user_id)
+    keystoneclient(request, admin=True).users.delete(user_id)
 
 
 @profiler.trace
@@ -513,9 +523,9 @@ def user_update(request, user, **data):
 def user_update_enabled(request, user, enabled):
     manager = keystoneclient(request, admin=True).users
     if VERSIONS.active < 3:
-        return manager.update_enabled(user, enabled)
+        manager.update_enabled(user, enabled)
     else:
-        return manager.update(user, enabled=enabled)
+        manager.update(user, enabled=enabled)
 
 
 @profiler.trace
@@ -527,9 +537,9 @@ def user_update_password(request, user, password, admin=True):
 
     manager = keystoneclient(request, admin=admin).users
     if VERSIONS.active < 3:
-        return manager.update_password(user, password)
+        manager.update_password(user, password)
     else:
-        return manager.update(user, password=password)
+        manager.update(user, password=password)
 
 
 def user_verify_admin_password(request, admin_password):
@@ -556,10 +566,11 @@ def user_verify_admin_password(request, admin_password):
 @profiler.trace
 def user_update_own_password(request, origpassword, password):
     client = keystoneclient(request, admin=False)
-    client.user_id = request.user.id
     if VERSIONS.active < 3:
+        client.user_id = request.user.id
         return client.users.update_own_password(origpassword, password)
     else:
+        client.users.client.session.auth.user_id = request.user.id
         return client.users.update_password(origpassword, password)
 
 
@@ -708,7 +719,7 @@ def role_update(request, role_id, name=None):
 @profiler.trace
 def role_delete(request, role_id):
     manager = keystoneclient(request, admin=True).roles
-    return manager.delete(role_id)
+    manager.delete(role_id)
 
 
 @profiler.trace
@@ -806,10 +817,10 @@ def add_tenant_user_role(request, project=None, user=None, role=None,
     """Adds a role for a user on a tenant."""
     manager = keystoneclient(request, admin=True).roles
     if VERSIONS.active < 3:
-        return manager.add_user_role(user, role, project)
+        manager.add_user_role(user, role, project)
     else:
-        return manager.grant(role, user=user, project=project,
-                             group=group, domain=domain)
+        manager.grant(role, user=user, project=project,
+                      group=group, domain=domain)
 
 
 @profiler.trace
@@ -1072,3 +1083,37 @@ def protocol_delete(request, identity_provider, protocol):
 def protocol_list(request, identity_provider):
     manager = keystoneclient(request).federation.protocols
     return manager.list(identity_provider)
+
+
+@profiler.trace
+def application_credential_list(request, filters=None):
+    user = request.user.id
+    manager = keystoneclient(request).application_credentials
+    return manager.list(user=user, **filters)
+
+
+@profiler.trace
+def application_credential_get(request, application_credential_id):
+    user = request.user.id
+    manager = keystoneclient(request).application_credentials
+    return manager.get(application_credential=application_credential_id,
+                       user=user)
+
+
+@profiler.trace
+def application_credential_delete(request, application_credential_id):
+    user = request.user.id
+    manager = keystoneclient(request).application_credentials
+    return manager.delete(application_credential=application_credential_id,
+                          user=user)
+
+
+@profiler.trace
+def application_credential_create(request, name, secret=None,
+                                  description=None, expires_at=None,
+                                  roles=None, unrestricted=False):
+    user = request.user.id
+    manager = keystoneclient(request).application_credentials
+    return manager.create(name=name, user=user, secret=secret,
+                          description=description, expires_at=expires_at,
+                          roles=roles, unrestricted=unrestricted)
