@@ -35,6 +35,7 @@ import six
 from horizon import exceptions
 from horizon import messages
 from horizon.utils.memoized import memoized
+from horizon.utils.memoized import memoized_with_request
 from openstack_dashboard.api import base
 from openstack_dashboard.api import nova
 from openstack_dashboard.contrib.developer.profiler import api as profiler
@@ -116,6 +117,36 @@ class Subnet(NeutronAPIDictWrapper):
         super(Subnet, self).__init__(apidict)
 
 
+AUTO_ALLOCATE_ID = '__auto_allocate__'
+
+
+class PreAutoAllocateNetwork(Network):
+    def __init__(self, request):
+        tenant_id = request.user.tenant_id
+        auto_allocated_subnet = Subnet({
+            'name': 'auto_allocated_subnet',
+            'id': AUTO_ALLOCATE_ID,
+            'network_id': 'auto',
+            'tenant_id': tenant_id,
+            # The following two fields are fake so that Subnet class
+            # and the network topology view work without errors.
+            'ip_version': 4,
+            'cidr': '0.0.0.0/0',
+        })
+        auto_allocated_network = {
+            'name': 'auto_allocated_network',
+            'description': 'Network to be allocated automatically',
+            'id': AUTO_ALLOCATE_ID,
+            'status': 'ACTIVE',
+            'admin_state_up': True,
+            'shared': False,
+            'router:external': False,
+            'subnets': [auto_allocated_subnet],
+            'tenant_id': tenant_id,
+        }
+        super(PreAutoAllocateNetwork, self).__init__(auto_allocated_network)
+
+
 class Trunk(NeutronAPIDictWrapper):
     """Wrapper for neutron trunks."""
 
@@ -249,6 +280,7 @@ class SecurityGroupRule(NeutronAPIDictWrapper):
             'ip_protocol': sgr['protocol'],
             'from_port': sgr['port_range_min'],
             'to_port': sgr['port_range_max'],
+            'description': sgr.get('description', '')
         }
         cidr = sgr['remote_ip_prefix']
         rule['ip_range'] = {'cidr': cidr} if cidr else {}
@@ -388,7 +420,7 @@ class SecurityGroupManager(object):
     def rule_create(self, parent_group_id,
                     direction=None, ethertype=None,
                     ip_protocol=None, from_port=None, to_port=None,
-                    cidr=None, group_id=None):
+                    cidr=None, group_id=None, description=None):
         """Create a new security group rule.
 
         :param parent_group_id: security group id a rule is created to
@@ -410,15 +442,17 @@ class SecurityGroupManager(object):
         if isinstance(ip_protocol, int) and ip_protocol < 0:
             ip_protocol = None
 
-        body = {'security_group_rule':
-                {'security_group_id': parent_group_id,
-                 'direction': direction,
-                 'ethertype': ethertype,
-                 'protocol': ip_protocol,
-                 'port_range_min': from_port,
-                 'port_range_max': to_port,
-                 'remote_ip_prefix': cidr,
-                 'remote_group_id': group_id}}
+        params = {'security_group_id': parent_group_id,
+                  'direction': direction,
+                  'ethertype': ethertype,
+                  'protocol': ip_protocol,
+                  'port_range_min': from_port,
+                  'port_range_max': to_port,
+                  'remote_ip_prefix': cidr,
+                  'remote_group_id': group_id}
+        if description is not None:
+            params['description'] = description
+        body = {'security_group_rule': params}
         try:
             rule = self.client.create_security_group_rule(body)
         except neutron_exc.OverQuotaClient:
@@ -460,7 +494,7 @@ class SecurityGroupManager(object):
 
 class FloatingIp(base.APIDictWrapper):
     _attrs = ['id', 'ip', 'fixed_ip', 'port_id', 'instance_id',
-              'instance_type', 'pool']
+              'instance_type', 'pool', 'dns_domain', 'dns_name']
 
     def __init__(self, fip):
         fip['ip'] = fip['floating_ip_address']
@@ -599,6 +633,10 @@ class FloatingIpManager(object):
             create_dict['floating_ip_address'] = params['floating_ip_address']
         if 'description' in params:
             create_dict['description'] = params['description']
+        if 'dns_domain' in params:
+            create_dict['dns_domain'] = params['dns_domain']
+        if 'dns_name' in params:
+            create_dict['dns_name'] = params['dns_name']
         fip = self.client.create_floatingip(
             {'floatingip': create_dict}).get('floatingip')
         self._set_instance_info(fip)
@@ -762,13 +800,22 @@ def get_ipver_str(ip_version):
     return IP_VERSION_DICT.get(ip_version, '')
 
 
-@memoized
-def neutronclient(request):
+def get_auth_params_from_request(request):
+    return (
+        request.user.token.id,
+        base.url_for(request, 'network'),
+        base.url_for(request, 'identity')
+    )
+
+
+@memoized_with_request(get_auth_params_from_request)
+def neutronclient(request_auth_params):
+    token_id, neutron_url, auth_url = request_auth_params
     insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
     cacert = getattr(settings, 'OPENSTACK_SSL_CACERT', None)
-    c = neutron_client.Client(token=request.user.token.id,
-                              auth_url=base.url_for(request, 'identity'),
-                              endpoint_url=base.url_for(request, 'network'),
+    c = neutron_client.Client(token=token_id,
+                              auth_url=auth_url,
+                              endpoint_url=neutron_url,
                               insecure=insecure, ca_cert=cacert)
     return c
 
@@ -978,8 +1025,35 @@ def network_list(request, **params):
     return [Network(n) for n in networks]
 
 
+def _is_auto_allocated_network_supported(request):
+    try:
+        neutron_auto_supported = is_service_enabled(
+            request, 'enable_auto_allocated_network',
+            'auto-allocated-topology', default=False)
+    except Exception:
+        exceptions.handle(request, _('Failed to check if neutron supports '
+                                     '"auto_allocated_network".'))
+        neutron_auto_supported = False
+    if not neutron_auto_supported:
+        return False
+
+    try:
+        # server_create needs to support both features,
+        # so we need to pass both features here.
+        nova_auto_supported = nova.is_feature_available(
+            request, ("instance_description",
+                      "auto_allocated_network"))
+    except Exception:
+        exceptions.handle(request, _('Failed to check if nova supports '
+                                     '"auto_allocated_network".'))
+        nova_auto_supported = False
+
+    return nova_auto_supported
+
+
 @profiler.trace
 def network_list_for_tenant(request, tenant_id, include_external=False,
+                            include_pre_auto_allocate=False,
                             **params):
     """Return a network list available for the tenant.
 
@@ -1005,6 +1079,12 @@ def network_list_for_tenant(request, tenant_id, include_external=False,
         # In the current Neutron API, there is no way to retrieve
         # both owner networks and public networks in a single API call.
         networks += network_list(request, shared=True, **params)
+
+    # Hack for auto allocated network
+    if include_pre_auto_allocate and not networks:
+        if _is_auto_allocated_network_supported(request):
+            networks.append(PreAutoAllocateNetwork(request))
+
     params['router:external'] = params.get('router:external', True)
     if params['router:external'] and include_external:
         if shared is not None:
@@ -1582,10 +1662,10 @@ def security_group_update(request, sg_id, name, desc):
 def security_group_rule_create(request, parent_group_id,
                                direction, ethertype,
                                ip_protocol, from_port, to_port,
-                               cidr, group_id):
+                               cidr, group_id, description=None):
     return SecurityGroupManager(request).rule_create(
         parent_group_id, direction, ethertype, ip_protocol,
-        from_port, to_port, cidr, group_id)
+        from_port, to_port, cidr, group_id, description)
 
 
 def security_group_rule_delete(request, sgr_id):
@@ -1706,10 +1786,14 @@ def _server_get_addresses(request, server, ports, floating_ips, network_names):
 
 
 @profiler.trace
-@memoized
-def list_extensions(request):
+@memoized_with_request(neutronclient)
+def list_extensions(neutron_api):
+    """List neutron extensions.
+
+    :param request: django request object
+    """
     try:
-        extensions_list = neutronclient(request).list_extensions()
+        extensions_list = neutron_api.list_extensions()
     except exceptions.ServiceCatalogException:
         return {}
     if 'extensions' in extensions_list:
@@ -1719,10 +1803,13 @@ def list_extensions(request):
 
 
 @profiler.trace
-@memoized
 def is_extension_supported(request, extension_alias):
-    extensions = list_extensions(request)
+    """Check if a specified extension is supported.
 
+    :param request: django request object
+    :param extension_alias: neutron extension alias
+    """
+    extensions = list_extensions(request)
     for extension in extensions:
         if extension['alias'] == extension_alias:
             return True
@@ -1736,8 +1823,8 @@ def is_enabled_by_config(name, default=True):
 
 
 @memoized
-def is_service_enabled(request, config_name, ext_name):
-    return (is_enabled_by_config(config_name) and
+def is_service_enabled(request, config_name, ext_name, default=True):
+    return (is_enabled_by_config(config_name, default) and
             is_extension_supported(request, ext_name))
 
 
