@@ -22,6 +22,7 @@ from __future__ import absolute_import
 
 import collections
 import logging
+from operator import attrgetter
 
 from django.conf import settings
 from django.utils.translation import ugettext_lazy as _
@@ -69,6 +70,10 @@ def is_feature_available(request, features):
     return bool(get_microversion(request, features))
 
 
+class VolumeMultiattachNotSupported(horizon_exceptions.HorizonException):
+    status_code = 400
+
+
 class VNCConsole(base.APIDictWrapper):
     """Wrapper for the "console" dictionary.
 
@@ -246,12 +251,6 @@ class QuotaSet(base.QuotaSet):
         "security_group_rules",
     }
 
-    def __init__(self, apiresource=None):
-        super(QuotaSet, self).__init__(apiresource)
-        for name in self.ignore_quotas:
-            if name in self.items:
-                del self.items[name]
-
 
 def get_auth_params_from_request(request):
     """Extracts properties needed by novaclient call from the request object.
@@ -373,8 +372,7 @@ def flavor_list(request, is_public=True, get_extras=False):
 
 
 @profiler.trace
-def update_pagination(entities, page_size, marker, sort_dir, sort_key,
-                      reversed_order):
+def update_pagination(entities, page_size, marker, reversed_order=False):
     has_more_data = has_prev_data = False
     if len(entities) > page_size:
         has_more_data = True
@@ -390,9 +388,7 @@ def update_pagination(entities, page_size, marker, sort_dir, sort_key,
 
     # restore the original ordering here
     if reversed_order:
-        entities = sorted(entities, key=lambda entity:
-                          (getattr(entity, sort_key) or '').lower(),
-                          reverse=(sort_dir == 'asc'))
+        entities.reverse()
 
     return entities, has_more_data, has_prev_data
 
@@ -416,7 +412,7 @@ def flavor_list_paged(request, is_public=True, get_extras=False, marker=None,
                                                    sort_key=sort_key,
                                                    sort_dir=sort_dir)
         flavors, has_more_data, has_prev_data = update_pagination(
-            flavors, page_size, marker, sort_dir, sort_key, reversed_order)
+            flavors, page_size, marker, reversed_order)
     else:
         flavors = novaclient(request).flavors.list(is_public=is_public)
 
@@ -513,10 +509,27 @@ def server_create(request, name, image, flavor, key_name, user_data,
                   availability_zone=None, instance_count=1, admin_pass=None,
                   disk_config=None, config_drive=None, meta=None,
                   scheduler_hints=None, description=None):
+    microversion = get_microversion(request, ("instance_description",
+                                              "auto_allocated_network"))
+    nova_client = novaclient(request, version=microversion)
+
+    # NOTE(amotoki): Handling auto allocated network
+    # Nova API 2.37 or later, it accepts a special string 'auto' for nics
+    # which means nova uses a network that is available for a current project
+    # if one exists and otherwise it creates a network automatically.
+    # This special handling is processed here as JS side assumes 'nics'
+    # is a list and it is easiest to handle it here.
+    if nics:
+        is_auto_allocate = any(nic.get('net-id') == '__auto_allocate__'
+                               for nic in nics)
+        if is_auto_allocate:
+            nics = 'auto'
+
     kwargs = {}
     if description is not None:
         kwargs['description'] = description
-    return Server(get_novaclient_with_instance_desc(request).servers.create(
+
+    return Server(nova_client.servers.create(
         name.strip(), image, flavor, userdata=user_data,
         security_groups=security_groups,
         key_name=key_name, block_device_mapping=block_device_mapping,
@@ -530,6 +543,14 @@ def server_create(request, name, image, flavor, key_name, user_data,
 @profiler.trace
 def server_delete(request, instance_id):
     novaclient(request).servers.delete(instance_id)
+    # Session is available and consistent for the current view
+    # among Horizon django servers even in load-balancing setup,
+    # so only the view listing the servers will recognize it as
+    # own DeleteInstance action performed. Note that dict is passed
+    # by reference in python. Quote from django's developer manual:
+    # " You can read it and write to request.session at any point
+    #   in your view. You can edit it multiple times."
+    request.session['server_deleted'] = instance_id
 
 
 def get_novaclient_with_locked_status(request):
@@ -549,33 +570,80 @@ def server_get(request, instance_id):
 
 
 @profiler.trace
-def server_list(request, search_opts=None, detailed=True):
+def server_list_paged(request,
+                      search_opts=None,
+                      detailed=True,
+                      sort_dir="desc"):
+    has_more_data = False
+    has_prev_data = False
     nova_client = get_novaclient_with_locked_status(request)
     page_size = utils.get_page_size(request)
-    paginate = False
-    if search_opts is None:
-        search_opts = {}
-    elif 'paginate' in search_opts:
-        paginate = search_opts.pop('paginate')
-        if paginate:
-            search_opts['limit'] = page_size + 1
+    search_opts = {} if search_opts is None else search_opts
+    marker = search_opts.get('marker', None)
 
-    all_tenants = search_opts.get('all_tenants', False)
-    if all_tenants:
-        search_opts['all_tenants'] = True
-    else:
+    if not search_opts.get('all_tenants', False):
         search_opts['project_id'] = request.user.tenant_id
 
-    servers = [Server(s, request)
-               for s in nova_client.servers.list(detailed, search_opts)]
+    if search_opts.pop('paginate', False):
+        reversed_order = sort_dir is "asc"
+        LOG.debug("Notify received on deleted server: %r",
+                  ('server_deleted' in request.session))
+        deleted = request.session.pop('server_deleted',
+                                      None)
+        view_marker = 'possibly_deleted' if deleted and marker else 'ok'
+        search_opts['marker'] = deleted if deleted else marker
+        search_opts['limit'] = page_size + 1
+        # NOTE(amotoki): It looks like the 'sort_keys' must be unique to make
+        # the pagination in the nova API works as expected. Multiple servers
+        # can have a same 'created_at' as its resolution is a second.
+        # To ensure the uniqueness we add 'uuid' to the sort keys.
+        # 'display_name' is added before 'uuid' to list servers in the
+        # alphabetical order.
+        sort_keys = ['created_at', 'display_name', 'uuid']
 
-    has_more_data = False
-    if paginate and len(servers) > page_size:
-        servers.pop(-1)
-        has_more_data = True
-    elif paginate and len(servers) == getattr(settings, 'API_RESULT_LIMIT',
-                                              1000):
-        has_more_data = True
+        servers = [Server(s, request)
+                   for s in nova_client.servers.list(detailed, search_opts,
+                                                     sort_keys=sort_keys,
+                                                     sort_dirs=[sort_dir] * 3)]
+
+        if view_marker == 'possibly_deleted':
+            if len(servers) == 0:
+                view_marker = 'head_deleted'
+                reversed_order = False
+                servers = [Server(s, request)
+                           for s in
+                           nova_client.servers.list(detailed,
+                                                    search_opts,
+                                                    sort_keys=sort_keys,
+                                                    sort_dirs=['desc'] * 3)]
+            if not servers:
+                view_marker = 'tail_deleted'
+                reversed_order = True
+                servers = [Server(s, request)
+                           for s in
+                           nova_client.servers.list(detailed,
+                                                    search_opts,
+                                                    sort_keys=sort_keys,
+                                                    sort_dirs=['asc'] * 3)]
+        (servers, has_more_data, has_prev_data) = update_pagination(
+            servers, page_size, marker, reversed_order)
+        has_prev_data = (False
+                         if view_marker == 'head_deleted'
+                         else has_prev_data)
+        has_more_data = (False
+                         if view_marker == 'tail_deleted'
+                         else has_more_data)
+    else:
+        servers = [Server(s, request)
+                   for s in nova_client.servers.list(detailed, search_opts)]
+    return (servers, has_more_data, has_prev_data)
+
+
+@profiler.trace
+def server_list(request, search_opts=None, detailed=True):
+    (servers, has_more_data, _) = server_list_paged(request,
+                                                    search_opts,
+                                                    detailed)
     return (servers, has_more_data)
 
 
@@ -626,9 +694,12 @@ def server_reboot(request, instance_id, soft_reboot=False):
 
 @profiler.trace
 def server_rebuild(request, instance_id, image_id, password=None,
-                   disk_config=None):
-    return novaclient(request).servers.rebuild(instance_id, image_id,
-                                               password, disk_config)
+                   disk_config=None, description=None):
+    kwargs = {}
+    if description:
+        kwargs['description'] = description
+    return get_novaclient_with_instance_desc(request).servers.rebuild(
+        instance_id, image_id, password, disk_config, **kwargs)
 
 
 @profiler.trace
@@ -794,9 +865,20 @@ def get_password(request, instance_id, private_key=None):
 
 @profiler.trace
 def instance_volume_attach(request, volume_id, instance_id, device):
-    return novaclient(request).volumes.create_server_volume(instance_id,
-                                                            volume_id,
-                                                            device)
+    from openstack_dashboard.api import cinder
+    # If we have a multiattach volume, we need to use microversion>=2.60.
+    volume = cinder.volume_get(request, volume_id)
+    if volume.multiattach:
+        version = get_microversion(request, 'multiattach')
+        if version:
+            client = novaclient(request, version)
+        else:
+            raise VolumeMultiattachNotSupported(
+                _('Multiattach volumes are not yet supported.'))
+    else:
+        client = novaclient(request)
+    return client.volumes.create_server_volume(
+        instance_id, volume_id, device)
 
 
 @profiler.trace
@@ -923,7 +1005,9 @@ def tenant_absolute_limits(request, reserved=False, tenant_id=None):
 
 @profiler.trace
 def availability_zone_list(request, detailed=False):
-    return novaclient(request).availability_zones.list(detailed=detailed)
+    zones = novaclient(request).availability_zones.list(detailed=detailed)
+    zones.sort(key=attrgetter('zoneName'))
+    return zones
 
 
 @profiler.trace
@@ -931,6 +1015,25 @@ def server_group_list(request):
     return novaclient(request).server_groups.list()
 
 
+@profiler.trace
+def server_group_create(request, **kwargs):
+    microversion = get_microversion(request, "servergroup_soft_policies")
+    return novaclient(request, version=microversion).server_groups.create(
+        **kwargs)
+
+
+@profiler.trace
+def server_group_delete(request, servergroup_id):
+    novaclient(request).server_groups.delete(servergroup_id)
+
+
+@profiler.trace
+def server_group_get(request, servergroup_id):
+    microversion = get_microversion(request, "servergroup_user_info")
+    return novaclient(request, version=microversion).server_groups.get(
+        servergroup_id)
+
+
 @profiler.trace
 def service_list(request, binary=None):
     return novaclient(request).services.list(binary=binary)
@@ -976,7 +1079,7 @@ def aggregate_get(request, aggregate_id):
 
 @profiler.trace
 def aggregate_update(request, aggregate_id, values):
-    return novaclient(request).aggregates.update(aggregate_id, values)
+    novaclient(request).aggregates.update(aggregate_id, values)
 
 
 @profiler.trace
@@ -986,12 +1089,12 @@ def aggregate_set_metadata(request, aggregate_id, metadata):
 
 @profiler.trace
 def add_host_to_aggregate(request, aggregate_id, host):
-    return novaclient(request).aggregates.add_host(aggregate_id, host)
+    novaclient(request).aggregates.add_host(aggregate_id, host)
 
 
 @profiler.trace
 def remove_host_from_aggregate(request, aggregate_id, host):
-    return novaclient(request).aggregates.remove_host(aggregate_id, host)
+    novaclient(request).aggregates.remove_host(aggregate_id, host)
 
 
 @profiler.trace
