@@ -14,7 +14,7 @@
 
 import logging
 
-from django.core.urlresolvers import reverse
+from django.urls import reverse
 from django.utils.translation import ugettext_lazy as _
 
 from horizon import exceptions
@@ -69,8 +69,8 @@ class AddInterface(forms.SelfHandlingForm):
                          '%s%s (%s)' % (net_name, subnet.cidr,
                                         subnet.name or subnet.id))
                         for subnet in n['subnets']
-                        if subnet.id not in router_subnet_ids
-                        and subnet.gateway_ip]
+                        if (subnet.id not in router_subnet_ids and
+                            subnet.gateway_ip)]
         if choices:
             choices.insert(0, ("", _("Select Subnet")))
         else:
