@@ -13,10 +13,10 @@
 #    under the License.
 
 from django.conf import settings
-from django.core.urlresolvers import NoReverseMatch
-from django.core.urlresolvers import reverse
 from django.http import HttpResponse
 from django.template import defaultfilters as filters
+from django.urls import NoReverseMatch
+from django.urls import reverse
 from django.utils import html
 from django.utils.http import urlencode
 from django.utils import safestring
@@ -119,7 +119,9 @@ class DeleteVolume(VolumePolicyTargetMixin, tables.DeleteAction):
             # Can't delete volume if part of consistency group
             if getattr(volume, 'consistencygroup_id', None):
                 return False
-
+            # Can't delete volume if part of volume group
+            if getattr(volume, 'group_id', None):
+                return False
             return (volume.status in DELETABLE_STATES and
                     not getattr(volume, 'has_snapshot', False))
         return True
@@ -141,10 +143,10 @@ class CreateVolume(tables.LinkAction):
     def allowed(self, request, volume=None):
         limits = api.cinder.tenant_absolute_limits(request)
 
-        gb_available = (limits.get('maxTotalVolumeGigabytes', float("inf"))
-                        - limits.get('totalGigabytesUsed', 0))
-        volumes_available = (limits.get('maxTotalVolumes', float("inf"))
-                             - limits.get('totalVolumesUsed', 0))
+        gb_available = (limits.get('maxTotalVolumeGigabytes', float("inf")) -
+                        limits.get('totalGigabytesUsed', 0))
+        volumes_available = (limits.get('maxTotalVolumes', float("inf")) -
+                             limits.get('totalVolumesUsed', 0))
 
         if gb_available <= 0 or volumes_available <= 0:
             if "disabled" not in self.classes:
@@ -188,12 +190,12 @@ class EditAttachments(tables.LinkAction):
             project_id = getattr(volume, "os-vol-tenant-attr:tenant_id", None)
             attach_allowed = \
                 policy.check((("compute",
-                             "os_compute_api:servers:attach_volume"),),
+                             "os_compute_api:os-volumes-attachments:create"),),
                              request,
                              {"project_id": project_id})
             detach_allowed = \
                 policy.check((("compute",
-                             "os_compute_api:servers:detach_volume"),),
+                             "os_compute_api:os-volumes-attachments:delete"),),
                              request,
                              {"project_id": project_id})
 
@@ -217,8 +219,8 @@ class CreateSnapshot(VolumePolicyTargetMixin, tables.LinkAction):
             exceptions.handle(request, _('Unable to retrieve tenant limits.'))
             limits = {}
 
-        snapshots_available = (limits.get('maxTotalSnapshots', float("inf"))
-                               - limits.get('totalSnapshotsUsed', 0))
+        snapshots_available = (limits.get('maxTotalSnapshots', float("inf")) -
+                               limits.get('totalSnapshotsUsed', 0))
 
         if snapshots_available <= 0 and "disabled" not in self.classes:
             self.classes = [c for c in self.classes] + ['disabled']
@@ -339,6 +341,14 @@ class UpdateRow(tables.Row):
 
     def get_data(self, request, volume_id):
         volume = cinder.volume_get(request, volume_id)
+        if volume and getattr(volume, 'group_id', None):
+            try:
+                volume.group = cinder.group_get(request, volume.group_id)
+            except Exception:
+                exceptions.handle(request, _("Unable to retrieve group."))
+                volume.group = None
+        else:
+            volume.group = None
         return volume
 
 
@@ -393,6 +403,17 @@ class AttachmentColumn(tables.WrappingColumn):
         return safestring.mark_safe(", ".join(attachments))
 
 
+class GroupNameColumn(tables.WrappingColumn):
+    def get_raw_data(self, volume):
+        group = volume.group
+        return group.name_or_id if group else _("-")
+
+    def get_link_url(self, volume):
+        group = volume.group
+        if group:
+            return reverse(self.link, args=(group.id,))
+
+
 def get_volume_type(volume):
     return volume.volume_type if volume.volume_type != "None" else None
 
@@ -500,6 +521,10 @@ class VolumesTable(VolumesTableBase):
     name = tables.WrappingColumn("name",
                                  verbose_name=_("Name"),
                                  link="horizon:project:volumes:detail")
+    group = GroupNameColumn(
+        "name",
+        verbose_name=_("Group"),
+        link="horizon:project:volume_groups:detail")
     volume_type = tables.Column(get_volume_type,
                                 verbose_name=_("Type"))
     attachments = AttachmentColumn("attachments",
