@@ -16,30 +16,32 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import collections
-import copy
 from functools import wraps
 from importlib import import_module
 import logging
 import os
 import traceback
-import unittest
 
-import django
 from django.conf import settings
 from django.contrib.messages.storage import default_storage
 from django.core.handlers import wsgi
-from django.core import urlresolvers
-from django import http as http_request
 from django.test.client import RequestFactory
-from django.test import utils as django_test_utils
+from django.test import tag
+from django import urls
 from django.utils import http
 
 from cinderclient import client as cinder_client
 import glanceclient
 from keystoneclient.v2_0 import client as keystone_client
+# As of Rocky, we are in the process of removing mox usage.
+# To allow mox-free horizon plugins to consume the test helper,
+# mox import is now optional. If tests depends on mox,
+# mox (or mox3) must be declared in test-requirements.txt.
 import mock
-from mox3 import mox
+try:
+    from mox3 import mox
+except ImportError:
+    pass
 from neutronclient.v2_0 import client as neutron_client
 from novaclient import api_versions as nova_api_versions
 from novaclient.v2 import client as nova_client
@@ -63,6 +65,11 @@ LOG = logging.getLogger(__name__)
 # Makes output of failing mox tests much easier to read.
 wsgi.WSGIRequest.__repr__ = lambda self: "<class 'django.http.HttpRequest'>"
 
+# Shortcuts to avoid importing horizon_helpers and for backward compatibility.
+update_settings = horizon_helpers.update_settings
+IsA = horizon_helpers.IsA
+IsHttpRequest = horizon_helpers.IsHttpRequest
+
 
 def create_stubs(stubs_to_create=None):
     """decorator to simplify setting up multiple stubs at once via mox
@@ -119,13 +126,16 @@ def create_stubs(stubs_to_create=None):
     return inner_stub_out
 
 
-def create_mocks(target, methods):
+def create_mocks(target_methods):
     """decorator to simplify setting up multiple mocks at once
 
-    :param target: target object whose attribute(s) are patched.
-    :param methods: a list of methods to be patched using mock.
+    :param target_methods: a dict to define methods to be patched using mock.
 
-    Each element of methods argument can be a string or a tuple
+    A key of "target_methods" is a target object whose attribute(s) are
+    patched.
+
+    A value of "target_methods" is a list of methods to be patched
+    using mock. Each element of the list can be a string or a tuple
     consisting of two strings.
 
     A string specifies a method name of "target" object to be mocked.
@@ -133,7 +143,12 @@ def create_mocks(target, methods):
     can be accessed via 'mock_<method-name>' of the test class.
     For example, in case of::
 
-        @create_mocks(api.nova, ['server_list'])
+        @create_mocks({api.nova: ['server_list',
+                                  'flavor_list']})
+        def test_example(self):
+            ...
+            self.mock_server_list.return_value = ...
+            self.mock_flavar_list.side_effect = ...
 
     you can access the mocked method via "self.mock_server_list"
     inside a test class.
@@ -150,15 +165,21 @@ def create_mocks(target, methods):
 
     Example::
 
-        @create_mocks(
-            api.nova,
-            ['usage_get',
-             ('tenant_absolute_limits', 'nova_tenant_absolute_limits'),
-             'extension_supported'])
+        @create_mocks({
+            api.nova: [
+                'usage_get',
+                ('tenant_absolute_limits', 'nova_tenant_absolute_limits'),
+                'extension_supported',
+            ],
+            api.cinder: [
+                ('tenant_absolute_limits', 'cinder_tenant_absolute_limits'),
+            ],
+        })
         def test_example(self):
             ...
             self.mock_usage_get.return_value = ...
             self.mock_nova_tenant_absolute_limits.return_value = ...
+            self.mock_cinder_tenant_absolute_limits.return_value = ...
             ...
             self.mock_extension_supported.assert_has_calls(....)
 
@@ -166,15 +187,16 @@ def create_mocks(target, methods):
     def wrapper(function):
         @wraps(function)
         def wrapped(inst, *args, **kwargs):
-            for method in methods:
-                if isinstance(method, str):
-                    method_mocked = method
-                    attr_name = method
-                else:
-                    method_mocked = method[0]
-                    attr_name = method[1]
-                m = mock.patch.object(target, method_mocked)
-                setattr(inst, 'mock_%s' % attr_name, m.start())
+            for target, methods in target_methods.items():
+                for method in methods:
+                    if isinstance(method, str):
+                        method_mocked = method
+                        attr_name = method
+                    else:
+                        method_mocked = method[0]
+                        attr_name = method[1]
+                    m = mock.patch.object(target, method_mocked)
+                    setattr(inst, 'mock_%s' % attr_name, m.start())
             return function(inst, *args, **kwargs)
         return wrapped
     return wrapper
@@ -213,8 +235,6 @@ class RequestFactoryWithMessages(RequestFactory):
         return req
 
 
-@unittest.skipIf(os.environ.get('SKIP_UNITTESTS', False),
-                 "The SKIP_UNITTESTS env variable is set.")
 class TestCase(horizon_helpers.TestCase):
     """Specialized base test case class for Horizon.
 
@@ -226,6 +246,7 @@ class TestCase(horizon_helpers.TestCase):
       :class:`~openstack_dashboard.test.test_data.utils.TestData`
       for more information.
     * The ``mox`` mocking framework via ``self.mox``.
+      if ``use_mox`` attribute is set to True.
     * A set of request context data via ``self.context``.
     * A ``RequestFactory`` class which supports Django's ``contrib.messages``
       framework via ``self.factory``.
@@ -324,14 +345,10 @@ class TestCase(horizon_helpers.TestCase):
         Asserts that the given response issued a 302 redirect without
         processing the view which is redirected to.
         """
-        if django.VERSION >= (1, 9):
-            loc = six.text_type(response._headers.get('location', None)[1])
-            loc = http.urlunquote(loc)
-            expected_url = http.urlunquote(expected_url)
-            self.assertEqual(loc, expected_url)
-        else:
-            self.assertEqual(response._headers.get('location', None),
-                             ('Location', settings.TESTSERVER + expected_url))
+        loc = six.text_type(response._headers.get('location', None)[1])
+        loc = http.urlunquote(loc)
+        expected_url = http.urlunquote(expected_url)
+        self.assertEqual(loc, expected_url)
         self.assertEqual(response.status_code, 302)
 
     def assertNoFormErrors(self, response, context_name="form"):
@@ -427,7 +444,7 @@ class TestCase(horizon_helpers.TestCase):
     @staticmethod
     def mock_rest_request(**args):
         mock_args = {
-            'user.is_authenticated.return_value': True,
+            'user.is_authenticated': True,
             'is_ajax.return_value': True,
             'policy.check.return_value': True,
             'body': ''
@@ -435,6 +452,11 @@ class TestCase(horizon_helpers.TestCase):
         mock_args.update(args)
         return mock.Mock(**mock_args)
 
+    def assert_mock_multiple_calls_with_same_arguments(
+            self, mocked_method, count, expected_call):
+        self.assertEqual(count, mocked_method.call_count)
+        mocked_method.assert_has_calls([expected_call] * count)
+
 
 class BaseAdminViewTests(TestCase):
     """Sets an active user with the "admin" role.
@@ -464,8 +486,16 @@ class APITestCase(TestCase):
     For use with tests which deal with the underlying clients rather than
     stubbing out the openstack_dashboard.api.* methods.
     """
+
+    # NOTE: This test class depends on mox but does not declare use_mox = True
+    # to notify mox is no longer recommended.
+    # If a consumer would like to use this class, declare use_mox = True.
+
     def setUp(self):
         super(APITestCase, self).setUp()
+        LOG.warning("APITestCase has been deprecated in favor of mock usage "
+                    "and will be removed at the beginning of 'Stein' release. "
+                    "Please convert your to use APIMockTestCase instead.")
         utils.patch_middleware_get_user()
 
         def fake_keystoneclient(request, admin=False):
@@ -509,7 +539,16 @@ class APITestCase(TestCase):
         api.neutron.neutronclient = self._original_neutronclient
         api.cinder.cinderclient = self._original_cinderclient
 
+    def _warn_client(self, service, removal_version):
+        LOG.warning(
+            "APITestCase has been deprecated for %(service)s-related "
+            "tests and will be removerd in '%(removal_version)s' release. "
+            "Please convert your to use APIMockTestCase instead.",
+            {'service': service, 'removal_version': removal_version}
+        )
+
     def stub_novaclient(self):
+        self._warn_client('nova', 'Stein')
         if not hasattr(self, "novaclient"):
             self.mox.StubOutWithMock(nova_client, 'Client')
             # mock the api_version since MockObject.__init__ ignores it.
@@ -524,15 +563,14 @@ class APITestCase(TestCase):
         return self.novaclient
 
     def stub_cinderclient(self):
-        LOG.warning("APITestCase has been deprecated for Cinder-related "
-                    "tests and will be removerd in 'S' release. Please "
-                    "convert  your to use APIMockTestCase instead.")
+        self._warn_client('cinder', 'Stein')
         if not hasattr(self, "cinderclient"):
             self.mox.StubOutWithMock(cinder_client, 'Client')
             self.cinderclient = self.mox.CreateMock(cinder_client.Client)
         return self.cinderclient
 
     def stub_keystoneclient(self):
+        self._warn_client('keystone', 'Stein')
         if not hasattr(self, "keystoneclient"):
             self.mox.StubOutWithMock(keystone_client, 'Client')
             # NOTE(saschpe): Mock properties, MockObject.__init__ ignores them:
@@ -546,21 +584,21 @@ class APITestCase(TestCase):
         return self.keystoneclient
 
     def stub_glanceclient(self):
-        LOG.warning("APITestCase has been deprecated for Glance-related "
-                    "tests and will be removerd in 'S' release. Please "
-                    "convert  your to use APIMockTestCase instead.")
+        self._warn_client('glance', 'Stein')
         if not hasattr(self, "glanceclient"):
             self.mox.StubOutWithMock(glanceclient, 'Client')
             self.glanceclient = self.mox.CreateMock(glanceclient.Client)
         return self.glanceclient
 
     def stub_neutronclient(self):
+        self._warn_client('neutron', 'Stein')
         if not hasattr(self, "neutronclient"):
             self.mox.StubOutWithMock(neutron_client, 'Client')
             self.neutronclient = self.mox.CreateMock(neutron_client.Client)
         return self.neutronclient
 
     def stub_swiftclient(self, expected_calls=1):
+        self._warn_client('swift', 'Stein')
         if not hasattr(self, "swiftclient"):
             self.mox.StubOutWithMock(swift_client, 'Connection')
             self.swiftclient = self.mox.CreateMock(swift_client.Connection)
@@ -578,16 +616,11 @@ class APITestCase(TestCase):
         return self.swiftclient
 
 
-class APIMockTestCase(APITestCase):
-    def stub_cinderclient(self):
-        if not hasattr(self, "cinderclient"):
-            self.cinderclient = mock.Mock()
-        return self.cinderclient
+class APIMockTestCase(TestCase):
 
-    def stub_glanceclient(self):
-        if not hasattr(self, "glanceclient"):
-            self.glanceclient = mock.Mock()
-        return self.glanceclient
+    def setUp(self):
+        super(APIMockTestCase, self).setUp()
+        utils.patch_middleware_get_user()
 
 
 # Need this to test both Glance API V1 and V2 versions
@@ -601,8 +634,7 @@ class ResetImageAPIVersionMixin(object):
         super(ResetImageAPIVersionMixin, self).tearDown()
 
 
-@unittest.skipUnless(os.environ.get('WITH_SELENIUM', False),
-                     "The WITH_SELENIUM env variable is not set.")
+@tag('selenium')
 class SeleniumTestCase(horizon_helpers.SeleniumTestCase):
 
     def setUp(self):
@@ -713,36 +745,11 @@ class PluginTestCase(TestCase):
         to be re-calculated after registering new dashboards. Useful
         only for testing and should never be used on a live site.
         """
-        urlresolvers.clear_url_caches()
+        urls.clear_url_caches()
         moves.reload_module(import_module(settings.ROOT_URLCONF))
         base.Horizon._urls()
 
 
-class update_settings(django_test_utils.override_settings):
-    """override_settings which allows override an item in dict.
-
-    django original override_settings replaces a dict completely,
-    however OpenStack dashboard setting has many dictionary configuration
-    and there are test case where we want to override only one item in
-    a dictionary and keep other items in the dictionary.
-    This version of override_settings allows this if keep_dict is True.
-
-    If keep_dict False is specified, the original behavior of
-    Django override_settings is used.
-    """
-
-    def __init__(self, keep_dict=True, **kwargs):
-        if keep_dict:
-            for key, new_value in kwargs.items():
-                value = getattr(settings, key, None)
-                if (isinstance(new_value, collections.Mapping) and
-                        isinstance(value, collections.Mapping)):
-                    copied = copy.copy(value)
-                    copied.update(new_value)
-                    kwargs[key] = copied
-        super(update_settings, self).__init__(**kwargs)
-
-
 def mock_obj_to_dict(r):
     return mock.Mock(**{'to_dict.return_value': r})
 
@@ -752,18 +759,3 @@ def mock_factory(r):
     mocked = mock_obj_to_dict(r)
     mocked.configure_mock(**r)
     return mocked
-
-
-class IsA(object):
-    """Class to compare param is a specified class."""
-    def __init__(self, cls):
-        self.cls = cls
-
-    def __eq__(self, other):
-        return isinstance(other, self.cls)
-
-
-class IsHttpRequest(IsA):
-    """Class to compare param is django.http.HttpRequest."""
-    def __init__(self):
-        super(IsHttpRequest, self).__init__(http_request.HttpRequest)
