@@ -59,6 +59,7 @@ class APIDict(api_base.APIDictWrapper):
 
 
 class APIVersionTests(test.TestCase):
+
     def test_equal(self):
         version = api_base.Version('1.0')
         self.assertEqual(1, version)
@@ -105,6 +106,7 @@ class APIVersionTests(test.TestCase):
 
 # Wrapper classes that only define _attrs don't need extra testing.
 class APIResourceWrapperTests(test.TestCase):
+
     def test_get_attribute(self):
         resource = APIResource.get_instance()
         self.assertEqual('foo', resource.foo)
@@ -131,6 +133,7 @@ class APIResourceWrapperTests(test.TestCase):
 
 
 class APIDictWrapperTests(test.TestCase):
+
     # APIDict allows for both attribute access and dictionary style [element]
     # style access.  Test both
     def test_get_item(self):
@@ -198,6 +201,7 @@ class APIDictWrapperTests(test.TestCase):
 
 
 class ApiVersionTests(test.TestCase):
+
     def setUp(self):
         super(ApiVersionTests, self).setUp()
         self.previous_settings = settings.OPENSTACK_API_VERSIONS
@@ -232,7 +236,7 @@ class ApiVersionTests(test.TestCase):
             self.fail("ConfigurationError raised inappropriately.")
 
 
-class ApiHelperTests(test.TestCase):
+class ApiHelperTests(test.APITestCase):
     """Tests for functions that don't use one of the api objects."""
 
     def test_url_for(self):
