@@ -18,37 +18,21 @@
 
 from __future__ import absolute_import
 
-from keystoneclient.v2_0 import client as keystone_client
+import mock
 import six
 
 from openstack_dashboard import api
 from openstack_dashboard.test import helpers as test
 
 
-class FakeConnection(object):
-    pass
-
-
-class ClientConnectionTests(test.TestCase):
-    def setUp(self):
-        super(ClientConnectionTests, self).setUp()
-        self.mox.StubOutWithMock(keystone_client, "Client")
-        self.internal_url = api.base.url_for(self.request,
-                                             'identity',
-                                             endpoint_type='internalURL')
-        self.admin_url = api.base.url_for(self.request,
-                                          'identity',
-                                          endpoint_type='adminURL')
-        self.conn = FakeConnection()
-
-
-class RoleAPITests(test.APITestCase):
+class RoleAPITests(test.APIMockTestCase):
     def setUp(self):
         super(RoleAPITests, self).setUp()
         self.role = self.roles.member
         self.roles = self.roles.list()
 
-    def test_remove_tenant_user(self):
+    @mock.patch.object(api.keystone, 'keystoneclient')
+    def test_remove_tenant_user(self, mock_keystoneclient):
         """Tests api.keystone.remove_tenant_user
 
         Verifies that remove_tenant_user is called with the right arguments
@@ -57,34 +41,40 @@ class RoleAPITests(test.APITestCase):
         There are no assertions in this test because the checking is handled
         by mox in the VerifyAll() call in tearDown().
         """
-        keystoneclient = self.stub_keystoneclient()
+        keystoneclient = mock_keystoneclient.return_value
         tenant = self.tenants.first()
 
-        keystoneclient.roles = self.mox.CreateMockAnything()
-        keystoneclient.roles.roles_for_user(self.user.id,
-                                            tenant.id).AndReturn(self.roles)
-        for role in self.roles:
-            keystoneclient.roles.revoke(role.id,
-                                        domain=None,
-                                        group=None,
-                                        project=tenant.id,
-                                        user=self.user.id)
-        self.mox.ReplayAll()
+        keystoneclient.roles.roles_for_user.return_value = self.roles
+        keystoneclient.roles.revoke.side_effect = [None for role in self.roles]
+
         api.keystone.remove_tenant_user(self.request, tenant.id, self.user.id)
 
-    def test_get_default_role(self):
-        keystoneclient = self.stub_keystoneclient()
-        keystoneclient.roles = self.mox.CreateMockAnything()
-        keystoneclient.roles.list().AndReturn(self.roles)
-        self.mox.ReplayAll()
+        keystoneclient.roles.roles_for_user.assert_called_once_with(
+            self.user.id, tenant.id)
+        keystoneclient.roles.revoke.assert_has_calls(
+            [mock.call(role.id,
+                       domain=None,
+                       group=None,
+                       project=tenant.id,
+                       user=self.user.id)
+             for role in self.roles]
+        )
+
+    @mock.patch.object(api.keystone, 'keystoneclient')
+    def test_get_default_role(self, mock_keystoneclient):
+        keystoneclient = mock_keystoneclient.return_value
+        keystoneclient.roles.list.return_value = self.roles
+
         role = api.keystone.get_default_role(self.request)
         self.assertEqual(self.role, role)
         # Verify that a second call doesn't hit the API again,
         # (it would show up in mox as an unexpected method call)
         role = api.keystone.get_default_role(self.request)
 
+        keystoneclient.roles.list.assert_called_once_with()
 
-class ServiceAPITests(test.APITestCase):
+
+class ServiceAPITests(test.APIMockTestCase):
     def test_service_wrapper(self):
         catalog = self.service_catalog
         identity_data = api.base.get_service_from_catalog(catalog, "identity")
@@ -114,3 +104,16 @@ class ServiceAPITests(test.APITestCase):
         self.assertEqual("http://public.nova2.example.com:8774/v2",
                          service.public_url)
         self.assertEqual("int.nova2.example.com", service.host)
+
+
+class APIVersionTests(test.APIMockTestCase):
+    @mock.patch.object(api.keystone, 'keystoneclient')
+    def test_get_identity_api_version(self, mock_keystoneclient):
+        keystoneclient = mock_keystoneclient.return_value
+        endpoint_data = mock.Mock()
+        endpoint_data.api_version = (3, 10)
+        keystoneclient.session.get_endpoint_data.return_value = endpoint_data
+        api_version = api.keystone.get_identity_api_version(self.request)
+        keystoneclient.session.get_endpoint_data.assert_called_once_with(
+            service_type='identity')
+        self.assertEqual((3, 10), api_version)
