@@ -24,6 +24,7 @@ from openstack_dashboard.api import cinder
 from openstack_dashboard.api import neutron
 from openstack_dashboard.api import nova
 from openstack_dashboard.contrib.developer.profiler import api as profiler
+from openstack_dashboard.utils import futurist_utils
 
 
 LOG = logging.getLogger(__name__)
@@ -38,6 +39,8 @@ NOVA_COMPUTE_QUOTA_FIELDS = {
     "injected_file_path_bytes",
     "ram",
     "key_pairs",
+    "server_groups",
+    "server_group_members",
 }
 
 # We no longer supports nova-network, so network related quotas from nova
@@ -233,23 +236,33 @@ def get_tenant_quota_data(request, disabled_quotas=None, tenant_id=None):
 # It is confusing and makes the code complicated. They should be push away.
 # Check Identity Project panel and System Defaults panel too.
 @profiler.trace
-def get_disabled_quotas(request):
+def get_disabled_quotas(request, targets=None):
+    if targets:
+        candidates = set(targets)
+    else:
+        candidates = QUOTA_FIELDS
+
     # We no longer supports nova network, so we always disable
     # network related nova quota fields.
     disabled_quotas = set()
 
     # Cinder
-    if not cinder.is_volume_service_enabled(request):
-        disabled_quotas.update(CINDER_QUOTA_FIELDS)
+    if candidates & CINDER_QUOTA_FIELDS:
+        if not cinder.is_volume_service_enabled(request):
+            disabled_quotas.update(CINDER_QUOTA_FIELDS)
 
     # Neutron
-    if not base.is_service_enabled(request, 'network'):
+    if not (candidates & NEUTRON_QUOTA_FIELDS):
+        pass
+    elif not base.is_service_enabled(request, 'network'):
         disabled_quotas.update(NEUTRON_QUOTA_FIELDS)
     else:
-        if not neutron.is_extension_supported(request, 'security-group'):
+        if ({'security_group', 'security_group_rule'} & candidates and
+                not neutron.is_extension_supported(request, 'security-group')):
             disabled_quotas.update(['security_group', 'security_group_rule'])
 
-        if not neutron.is_router_enabled(request):
+        if ({'router', 'floatingip'} & candidates and
+                not neutron.is_router_enabled(request)):
             disabled_quotas.update(['router', 'floatingip'])
 
         try:
@@ -260,9 +273,13 @@ def get_disabled_quotas(request):
                           "quotas extension is enabled.")
 
     # Nova
-    if not (base.is_service_enabled(request, 'compute') and
-            nova.can_set_quotas()):
-        disabled_quotas.update(NOVA_QUOTA_FIELDS)
+    if candidates & NOVA_QUOTA_FIELDS:
+        if not (base.is_service_enabled(request, 'compute') and
+                nova.can_set_quotas()):
+            disabled_quotas.update(NOVA_QUOTA_FIELDS)
+
+    enabled_quotas = candidates - disabled_quotas
+    disabled_quotas = set(QUOTA_FIELDS) - enabled_quotas
 
     # There appear to be no glance quota fields currently
     return disabled_quotas
@@ -388,9 +405,6 @@ def _get_tenant_volume_usages(request, usages, disabled_quotas, tenant_id):
                              disabled_quotas)
 
 
-# TODO(amotoki): Merge tenant_quota_usages and tenant_limit_usages.
-# These two functions are similar. There seems no reason to have both.
-
 @profiler.trace
 @memoized
 def tenant_quota_usages(request, tenant_id=None, targets=None):
@@ -404,53 +418,20 @@ def tenant_quota_usages(request, tenant_id=None, targets=None):
     if not tenant_id:
         tenant_id = request.user.project_id
 
-    disabled_quotas = get_disabled_quotas(request)
+    disabled_quotas = get_disabled_quotas(request, targets)
     usages = QuotaUsage()
 
-    if targets:
-        if set(targets) - QUOTA_FIELDS:
-            raise ValueError('Unknown quota field names are included: %s'
-                             % set(targets) - QUOTA_FIELDS)
-        enabled_quotas = set(QUOTA_FIELDS) - disabled_quotas
-        enabled_quotas &= set(targets)
-        disabled_quotas = set(QUOTA_FIELDS) - enabled_quotas
-
-    _get_tenant_compute_usages(request, usages, disabled_quotas, tenant_id)
-    _get_tenant_network_usages(request, usages, disabled_quotas, tenant_id)
-    _get_tenant_volume_usages(request, usages, disabled_quotas, tenant_id)
+    futurist_utils.call_functions_parallel(
+        (_get_tenant_compute_usages,
+         [request, usages, disabled_quotas, tenant_id]),
+        (_get_tenant_network_usages,
+         [request, usages, disabled_quotas, tenant_id]),
+        (_get_tenant_volume_usages,
+         [request, usages, disabled_quotas, tenant_id]))
 
     return usages
 
 
-@profiler.trace
-def tenant_limit_usages(request):
-    # TODO(licostan): This method shall be removed from Quota module.
-    # ProjectUsage/BaseUsage maybe used instead on volume/image dashboards.
-    limits = {}
-
-    try:
-        if base.is_service_enabled(request, 'compute'):
-            limits.update(nova.tenant_absolute_limits(request, reserved=True))
-    except Exception:
-        msg = _("Unable to retrieve compute limit information.")
-        exceptions.handle(request, msg)
-
-    if cinder.is_volume_service_enabled(request):
-        try:
-            limits.update(cinder.tenant_absolute_limits(request))
-        except cinder.cinder_exception.ClientException:
-            msg = _("Unable to retrieve volume limit information.")
-            exceptions.handle(request, msg)
-
-    # TODO(amotoki): Support neutron quota details extensions
-    # which returns limit/usage/reserved per resource.
-    # Note that the data format is different from nova/cinder limit API.
-    # https://developer.openstack.org/
-    #   api-ref/network/v2/#quotas-details-extension-quota-details
-
-    return limits
-
-
 def enabled_quotas(request):
     """Returns the list of quotas available minus those that are disabled"""
     return QUOTA_FIELDS - get_disabled_quotas(request)
